//////////////////////////////////////////////////////////////////////////////
//
// This software module was originally developed by
//
// Dummy author
// Dummy company
//
// in the course of development of the MPEG-7 Experimentation Model.
//
// This software module is an implementation of a part of one or more MPEG-7
// Experimentation Model tools as specified by the MPEG-7 Requirements.
//
// ISO/IEC gives users of MPEG-7 free license to this software module or
// modifications thereof for use in hardware or software products claiming
// conformance to MPEG-7.
//
// Those intending to use this software module in hardware or software products
// are advised that its use may infringe existing patents. The original
// developer of this software module and his/her company, the subsequent
// editors and their companies, and ISO/IEC have no liability for use of this
// software module or modifications thereof in an implementation.
//
// Copyright is not released for non MPEG-7 conforming products. The
// organizations named above retain full right to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third
// parties from using the code for non MPEG-7 conforming products.
//
// Copyright (c) 1998-1999.
//
// This notice must be included in all copies or derivative works.
//
// Applicable File Name:  DummyTypeExtraction.cpp
//

#include <stdio.h>
#include "momusys.h"
#include "inter.h"
#include "intra.h"
#include "fifo.h"
#include "Descriptors.h"                     /* Naming Convention */
#include "ExtractionUtilities.h"
#include "readparam.h"

//static int initextractionwascalled=0;


//=============================================================================
	

using namespace XM;


const UUID DummyTypeExtractionInterface::myID = UUID("");
const char *DummyTypeExtractionInterface::myName =
"DummyTypeDescriptorExtractorInterface";

const UUID DummyTypeExtractionTool::myID = UUID("");
const char *DummyTypeExtractionTool::myName = "DummyTypeDescriptorExtractor";

const UUID DummyTypeExtractionInterfaceABC::myID = UUID();

//=============================================================================
DummyTypeExtractionInterfaceABC::DummyTypeExtractionInterfaceABC()
{
  SetInterfaceID(myID);
}

//=============================================================================
DummyTypeExtractionInterface::
DummyTypeExtractionInterface(DummyTypeExtractionTool* aTool):
m_ExtractionTool(aTool)
{
  SetInterfaceID(myID);
}

//----------------------------------------------------------------------------
DummyTypeExtractionInterface::~DummyTypeExtractionInterface()
{
}

//----------------------------------------------------------------------------
const UUID& DummyTypeExtractionInterface::GetInterfaceID(void)
{ return myID; }

//----------------------------------------------------------------------------
const char *DummyTypeExtractionInterface::GetName(void)
{ return myName; }


//----------------------------------------------------------------------------
void DummyTypeExtractionInterface::destroy(void)
{ delete m_ExtractionTool; }

//----------------------------------------------------------------------------
// This informs the extractor where the source data is coming from. This
// could be an audio file, image stream etc.
int DummyTypeExtractionInterface::SetSourceMedia(MultiMediaInterfaceABC* media)
{
  return m_ExtractionTool->SetSourceMedia(media);
}

//----------------------------------------------------------------------------
DummyTypeDescriptorInterfaceABC* DummyTypeExtractionInterface::
GetDescriptorInterface(void)

{
  return m_ExtractionTool->GetDescriptorInterface();
}

//----------------------------------------------------------------------------
int DummyTypeExtractionInterface::
SetDescriptorInterface(DummyTypeDescriptorInterfaceABC
		       *aDummyTypeDescriptorInterface)

{
  return m_ExtractionTool->
    SetDescriptorInterface(aDummyTypeDescriptorInterface);
}

//----------------------------------------------------------------------------
unsigned long DummyTypeExtractionInterface::InitExtracting(void)

{
  return(m_ExtractionTool->InitExtracting());
}

//----------------------------------------------------------------------------
unsigned long DummyTypeExtractionInterface::StartExtracting(void)

{
  return m_ExtractionTool->StartExtracting();
}

//----------------------------------------------------------------------------
unsigned long DummyTypeExtractionInterface::PostExtracting(void)

{
  return m_ExtractionTool->PostExtracting();
}

//=============================================================================
DummyTypeExtractionTool::DummyTypeExtractionTool():
m_Interface(this),
m_DescriptorInterface(NULL),
m_Media(NULL),
m_FrameCnt(0)

{
#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  /* create sub Extractions and
     connect them with descriptor if availabe*/
  SubDummyTypeAExtractionTool
    *aSubDummyTypeAExtraction= new SubDummyTypeAExtractionTool();
  SubDummyTypeBExtraction
    *aSubDummyTypeBExtraction= new SubDummyTypeBExtractionTool();

  /* connect sub Extractions with DummyType Extraction*/
  if (m_SubDummyTypeAExtraction)
    m_SubDummyTypeAExtraction=aSubDummyTypeAExtraction->GetInterface();
  //    SetSubDummyTypeAExtractionInterface(m_SubDummyTypeAExtraction->
  //					GetInterface());
  if (m_SubDummyTypeBExtraction)
    m_SubDummyTypeBExtraction=aSubDummyTypeBExtraction->GetInterface();
  //    SetSubDummyTypeBExtractionInterface(m_SubDummyTypeBExtraction->
  //					GetInterface());
#endif /* __HasSubTypes*/
  
  m_FrameCnt=0;
  SetExposedInterface(&m_Interface);
}

//=============================================================================
DummyTypeExtractionTool::
DummyTypeExtractionTool(DummyTypeDescriptorInterfaceABC
			*aDummyType):
m_Interface(this),
m_DescriptorInterface(NULL),
m_Media(NULL),
m_FrameCnt(0)

{
  /* create descriptor if it does not exist*/
  if (!aDummyType) {
    DummyTypeDescriptor *descriptor =
#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
      new DummyTypeDescriptor(0,0); /* one argument for each sub-component*/
#else
      new DummyTypeDescriptor;
#endif
      aDummyType=descriptor->GetInterface();
  }

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
#ifdef VERBOSE
  fprintf(stderr,"Connect sub descriptors\n");
#endif

  /* create sub Extraction and
     connect them with descriptor if availabe*/
  {
    SubDummyTypeAExtractionTool *aSubDummyTypeAExtraction;

    if (aDummyType->GetSubDummyTypeADescriptorInterface()) {
      aSubDummyTypeAExtraction= new
	SubDummyTypeAExtractionTool(aDummyType->
				    GetSubDummyTypeADescriptorInterface());
    }
    else { 
      aSubDummyTypeAExtraction= new SubDummyTypeAExtractionTool();
    }
    if (aSubDummyTypeAExtraction) {
      m_SubDummyTypeAExtraction=aSubDummyTypeAExtraction->GetInterface();
      //      SetSubDummyTypeAExtractionInterface(m_SubDummyTypeAExtraction);
    }
  }

  {
    SubDummyTypeBExtractionTool *aSubDummyTypeBExtraction;

    if (aDummyType->GetSubDummyTypeBDescriptorInterface()) {
      aSubDummyTypeBExtraction= new
	SubDummyTypeBExtractionTool(aDummyType->
				GetSubDummyTypeBDescriptorInterface());
    }
    else { 
      aSubDummyTypeBExtraction= new SubDummyTypeBExtractionTool();
    }
    if (aSubDummyTypeBExtraction) {
      m_SubDummyTypeBExtraction=aSubDummyTypeBExtraction->GetInterface();
      //      SetSubDummyTypeBExtractionInterface(m_SubDummyTypeBExtraction);
    }
  }
#endif /* __HasSubTypes*/

  /* connect descritors with Extraction*/
  SetDescriptorInterface(aDummyType);

  SetExposedInterface(&m_Interface);
}

//----------------------------------------------------------------------------
DummyTypeExtractionTool::~DummyTypeExtractionTool()
{
  // release descriptor*/
  if (m_DescriptorInterface)
    m_DescriptorInterface->release();
#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  if (m_SubDummyTypeAExtraction) m_SubDummyTypeAExtraction->destroy();
  if (m_SubDummyTypeBExtraction) m_SubDummyTypeBExtraction->destroy();
#endif /* __HasSubTypes*/
}

//----------------------------------------------------------------------------
bool DummyTypeExtractionTool::SupportsPush(void)
{ return true; }
//----------------------------------------------------------------------------
bool DummyTypeExtractionTool::SupportsPull(void)
{ return false; }

//----------------------------------------------------------------------------
// This informs the extractor where the source data is coming from. This
// could be an audio file, image stream etc. Here it's just a filename.
int DummyTypeExtractionTool::SetSourceMedia(MultiMediaInterfaceABC* media)
{
  if(media==NULL) return -1;
  m_Media=media;
#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  if (m_SubDummyTypeAExtraction)
    m_SubDummyTypeAExtraction->SetSourceMedia(media);
  if (m_SubDummyTypeBExtraction)
    m_SubDummyTypeBExtraction->SetSourceMedia(media);
#endif /* __HasSubTypes*/
  return 0;
}


//----------------------------------------------------------------------------
DummyTypeDescriptorInterfaceABC* DummyTypeExtractionTool::
GetDescriptorInterface(void)

{
  return m_DescriptorInterface;
}

//----------------------------------------------------------------------------
int DummyTypeExtractionTool::
SetDescriptorInterface(DummyTypeDescriptorInterfaceABC
		       *aDummyTypeDescriptorInterface)

{
  /* check if new value is different from old value*/
  if (m_DescriptorInterface == aDummyTypeDescriptorInterface) return 0;

  /* release old interface*/
  if (m_DescriptorInterface) m_DescriptorInterface->release();

  /* add new interface*/
  m_DescriptorInterface = aDummyTypeDescriptorInterface;
  if (m_DescriptorInterface) {
    m_DescriptorInterface->addref();

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
    /* conect sub descriptors with sub Extraction*/
    if ((m_DescriptorInterface->
	 GetSubDummyTypeADescriptorInterface()) &&
	m_SubDummyTypeAExtraction) {
      m_SubDummyTypeAExtraction->
	SetDescriptorInterface(m_DescriptorInterface->
			       GetSubDummyTypeADescriptorInterface());
    }
    if ((m_DescriptorInterface->
	 GetSubDummyTypeBDescriptorInterface()) &&
	m_SubDummyTypeBExtraction) {
      m_SubDummyTypeBExtraction->
	SetDescriptorInterface(m_DescriptorInterface->
			       GetSubDummyTypeBDescriptorInterface());
    }
#endif /* __HasSubTypes*/
  }
  else {
#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
    /* if no descriptor, release also descriptor interfaces from
       sub Extraction*/
    m_SubDummyTypeAExtraction->SetDescriptorInterface(0);
    m_SubDummyTypeBExtraction->SetDescriptorInterface(0);
#endif /* __HasSubTypes*/
    return -1;
  }

  return 0;
}

//----------------------------------------------------------------------------
// This initialises the extraction process. Bascially it should 
// reset all storage etc. However, as this hasn't got any storage
// it's a bit pointless.
unsigned long DummyTypeExtractionTool::InitExtracting(void)

{
  /* example for initializing extraction paramters*/
  m_DummyExtractionParameter=atoi(getpara("ParameterNameInFile","1"/*default
								     value*/));
  m_FrameCnt=0;//TOBEMODIFIED in your Application

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  /* init extraction of SubDummyTypeA description*/
  m_SubDummyTypeAExtraction->InitExtracting();

  /* init extraction of SubDummyTypeB description*/
  m_SubDummyTypeBExtraction->InitExtracting();
#endif /* __HasSubTypes*/

  return(0UL);
}

//----------------------------------------------------------------------------
// This uses the m_Media pointer to extract the relevant data and put it
// into the 
unsigned long DummyTypeExtractionTool::StartExtracting(void)

{
  // select media type you require and remove comment
  MomVop *ImageMedia;
  //  AudioType *AudioMedia;  // type of audio media still  needs to be defined

  // First check that it's all correctly initialised
  if (m_DescriptorInterface==NULL) return (unsigned long)-1;
  if(strcmp(m_DescriptorInterface->GetName(),
	    "DummyTypeDescriptionInterface") != 0) return (unsigned long)-1;


  if (m_Media==NULL) return (unsigned long)-1;
  ImageMedia=m_Media->GetImage();
  if (ImageMedia==NULL) return (unsigned long)-1;

  // Extract the value we want. For the dummy it's just the number of frames
  // in the Sequence multiplied with the extraction parameter value
  m_FrameCnt++;//TOBEMODIFIED in your Application
  const long int val=  m_FrameCnt*//TOBEMODIFIED in your Application
    m_DummyExtractionParameter;//TOBEMODIFIED in your Application
  
  // Increment frame Counter
  // write extracted data to Descriptor
  m_DescriptorInterface->SetDummyContents(val);//TOBEMODIFIED in your Application

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  /* extraction of SubDummyTypeA description*/
  m_SubDummyTypeAExtraction->StartExtracting();

  /* extraction of SubDummyTypeB description*/
  m_SubDummyTypeBExtraction->StartExtracting();
#endif /* __HasSubTypes*/

  // All done
  return 0;
}

//----------------------------------------------------------------------------
// This initialises the extraction process. Bascially it should 
// reset all storage etc. However, as this hasn't got any storage
// it's a bit pointless.
unsigned long DummyTypeExtractionTool::PostExtracting(void)

{
#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  /* post extraction of SubDummyTypeA description*/
  m_SubDummyTypeAExtraction->PostExtracting();

  /* post extraction of SubDummyTypeB description*/
  m_SubDummyTypeBExtraction->PostExtracting();
#endif /* __HasSubTypes*/

  // In this specific case you could also write the number of frames here
  //m_DescriptorInterface->SetDummyContents(m_FrameCnt*
  //					  m_DummyExtractionParameter);

  return(0UL);
}

//----------------------------------------------------------------------------
const UUID& DummyTypeExtractionTool::GetObjectID(void)
{ return myID; }

//----------------------------------------------------------------------------
const char *DummyTypeExtractionTool::GetName(void)
{ return myName; }

//----------------------------------------------------------------------------
DummyTypeExtractionInterfaceABC *DummyTypeExtractionTool::GetInterface(void)
{ return &m_Interface; }

