//////////////////////////////////////////////////////////////////////////////
//
// This software module was originally developed by
//
// Dummy author
// Dummy company
//
// in the course of development of the MPEG-7 Experimentation Model.
//
// This software module is an implementation of a part of one or more MPEG-7
// Experimentation Model tools as specified by the MPEG-7 Requirements.
//
// ISO/IEC gives users of MPEG-7 free license to this software module or
// modifications thereof for use in hardware or software products claiming
// conformance to MPEG-7.
//
// Those intending to use this software module in hardware or software products
// are advised that its use may infringe existing patents. The original
// developer of this software module and his/her company, the subsequent
// editors and their companies, and ISO/IEC have no liability for use of this
// software module or modifications thereof in an implementation.
//
// Copyright is not released for non MPEG-7 conforming products. The
// organizations named above retain full right to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third
// parties from using the code for non MPEG-7 conforming products.
//
// Copyright (c) 1998-1999.
//
// This notice must be included in all copies or derivative works.
//
// Applicable File Name:  DummyTypeSearch.cpp
//

#include <stdio.h>
#include <math.h>
#include <float.h>
#include "Descriptors.h"                     /* Naming Convention */
#include "SearchUtilities.h"

//=============================================================================
	

using namespace XM;


const UUID DummyTypeSearchInterface::myID = UUID("");
const char *DummyTypeSearchInterface::myName = "DummyTypeMatchingInterface";

const UUID DummyTypeSearchTool::myID = UUID("");
const char *DummyTypeSearchTool::myName = "DummyTypeMatchingTool";

const UUID DummyTypeSearchInterfaceABC::myID = UUID();

//=============================================================================
DummyTypeSearchInterfaceABC::DummyTypeSearchInterfaceABC()
{
  SetInterfaceID(myID);
}

//=============================================================================
DummyTypeSearchInterface::DummyTypeSearchInterface(DummyTypeSearchTool* aTool):
m_SearchTool(aTool)
{
  SetInterfaceID(myID);
}

//----------------------------------------------------------------------------
DummyTypeSearchInterface::~DummyTypeSearchInterface()
{
}

//----------------------------------------------------------------------------
const UUID& DummyTypeSearchInterface::GetInterfaceID(void)
{ return myID; }

//----------------------------------------------------------------------------
const char *DummyTypeSearchInterface::GetName(void)
{ return myName; }


//----------------------------------------------------------------------------
void DummyTypeSearchInterface::destroy(void)
{ delete m_SearchTool; }

//----------------------------------------------------------------------------
int DummyTypeSearchInterface::
SetRefDescriptorInterface(DummyTypeDescriptorInterfaceABC
		       *RefDummyTypeDescriptorInterface)

{
  return m_SearchTool->
    SetRefDescriptorInterface(RefDummyTypeDescriptorInterface);
}

//----------------------------------------------------------------------------
DummyTypeDescriptorInterfaceABC* DummyTypeSearchInterface::
GetQueryDescriptorInterface(void)

{
  return m_SearchTool->GetQueryDescriptorInterface();
}

//----------------------------------------------------------------------------
int DummyTypeSearchInterface::
SetQueryDescriptorInterface(DummyTypeDescriptorInterfaceABC
		       *QueryDummyTypeDescriptorInterface)

{
  return m_SearchTool->
    SetQueryDescriptorInterface(QueryDummyTypeDescriptorInterface);
}

//----------------------------------------------------------------------------
double DummyTypeSearchInterface::GetDistance(void)

{
  return m_SearchTool->GetDistance();
}

//=============================================================================
DummyTypeSearchTool::DummyTypeSearchTool():
m_Interface(this),
m_RefDescriptorInterface(0),
m_QueryDescriptorInterface(0)

{
#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  /* create sub Searchs and
     connect them with descriptor if availabe*/
  SubDummyTypeASearchTool *aSubDummyTypeASearch=
    new SubDummyTypeASearchTool();
  SubDummyTypeBSearchTool *aSubDummyTypeBSearch=
    new SubDummyTypeBSearchTool();

  /* connect sub Search with DummyTypeSearch */
  if (aSubDummyTypeASearch)
    m_SubDummyTypeASearch=aSubDummyTypeASearch->GetInterface();
  if (aSubDummyTypeBSearch)
    m_SubDummyTypeBSearch=aSubDummyTypeBSearch->GetInterface();

#endif /* __HasSubTypes*/
  SetExposedInterface(&m_Interface);
}

//=============================================================================
DummyTypeSearchTool::
DummyTypeSearchTool(DummyTypeDescriptorInterfaceABC
		    *aQueryDescriptorInterface):
m_Interface(this),
m_RefDescriptorInterface(0),
m_QueryDescriptorInterface(0)

{
  /* create descriptor if it does not exist*/
  if (!aQueryDescriptorInterface) {
    DummyTypeDescriptor *descriptor =
#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
      new DummyTypeDescriptor(0,0); /* one argument for each sub-component*/
#else
      new DummyTypeDescriptor;
#endif
    aQueryDescriptorInterface=descriptor->GetInterface();
  }

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
#ifdef VERBOSE
  fprintf(stderr,"Connect sub descriptors\n");
#endif

  /* create sub Search and
     connect them with descriptor if availabe*/
  {
    SubDummyTypeASearchTool *aSubDummyTypeASearch;

    if (aQueryDescriptorInterface->GetSubDummyTypeADescriptorInterface()) {
      aSubDummyTypeASearch= new
	SubDummyTypeASearchTool(aQueryDescriptorInterface->
				GetSubDummyTypeADescriptorInterface());
    }
    else { 
      aSubDummyTypeASearch= new SubDummyTypeASearchTool();
    }
    if (aSubDummyTypeASearch) {
      m_SubDummyTypeASearch=aSubDummyTypeASearch->GetInterface();
      //      SetSubDummyTypeASearchInterface(m_SubDummyTypeASearch);
    }
  }

  {
    SubDummyTypeBSearchTool *aSubDummyTypeBSearch;

    if (aQueryDescriptorInterface->GetSubDummyTypeBDescriptorInterface()) {
      aSubDummyTypeBSearch= new
	SubDummyTypeBSearchTool(aQueryDescriptorInterface->
				GetSubDummyTypeBDescriptorInterface());
    }
    else { 
      aSubDummyTypeBSearch= new SubDummyTypeBSearchTool();
    }
    if (aSubDummyTypeBSearch) {
      m_SubDummyTypeBSearch=aSubDummyTypeBSearch->GetInterface();
      //      SetSubDummyTypeBSearchInterface(m_SubDummyTypeBSearch);
    }
  }
#endif /* __HasSubTypes*/

  /* connect Descritors with Searchs*/
  SetQueryDescriptorInterface(aQueryDescriptorInterface);

  SetExposedInterface(&m_Interface);
}

//----------------------------------------------------------------------------
DummyTypeSearchTool::~DummyTypeSearchTool()
{
  /* release descriptor*/
  if (m_RefDescriptorInterface)
    m_RefDescriptorInterface->release();
  if (m_QueryDescriptorInterface)
    m_QueryDescriptorInterface->release();
#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  if (m_SubDummyTypeASearch) m_SubDummyTypeASearch->destroy();
  if (m_SubDummyTypeBSearch) m_SubDummyTypeBSearch->destroy();
#endif /* __HasSubTypes*/
}

//----------------------------------------------------------------------------
const UUID& DummyTypeSearchTool::GetObjectID(void)
{ return myID; }

//----------------------------------------------------------------------------
const char *DummyTypeSearchTool::GetName(void)
{ return myName; }

//----------------------------------------------------------------------------
bool DummyTypeSearchTool::SupportsPush(void)
{ return true; }

//----------------------------------------------------------------------------
bool DummyTypeSearchTool::SupportsPull(void)
{ return false; }

//----------------------------------------------------------------------------
int DummyTypeSearchTool::
SetRefDescriptorInterface(DummyTypeDescriptorInterfaceABC
		       *aRefDummyTypeDescriptorInterface)

{
  /* check if new value is different from old value*/
  if (m_RefDescriptorInterface == 
      aRefDummyTypeDescriptorInterface) return 0;
	
  /* release old interface*/
  if (m_RefDescriptorInterface) 
    m_RefDescriptorInterface->release();

  /* add new interface*/
  m_RefDescriptorInterface = 
    aRefDummyTypeDescriptorInterface;
  if (m_RefDescriptorInterface) {
    m_RefDescriptorInterface->addref();

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
    /* conect sub descriptors with sub Searchs*/
    if ((m_RefDescriptorInterface->
	 GetSubDummyTypeADescriptorInterface()) &&
	m_SubDummyTypeASearch) {
      m_SubDummyTypeASearch->
	SetRefDescriptorInterface(m_RefDescriptorInterface->
			       GetSubDummyTypeADescriptorInterface());
    }
    if ((m_RefDescriptorInterface->
	 GetSubDummyTypeBDescriptorInterface()) &&
	m_SubDummyTypeBSearch) {
      m_SubDummyTypeBSearch->
	SetRefDescriptorInterface(m_RefDescriptorInterface->
			       GetSubDummyTypeBDescriptorInterface());
    }
#endif /* __HasSubTypes*/
  }
  else {
#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
    /* if no descriptor, release also descriptor interfaces from
       sub Searchs*/
    m_SubDummyTypeASearch->SetRefDescriptorInterface(0);
    m_SubDummyTypeBSearch->SetRefDescriptorInterface(0);
#endif /* __HasSubTypes*/
    return -1;
  }

  return 0;
}

//----------------------------------------------------------------------------
DummyTypeDescriptorInterfaceABC* DummyTypeSearchTool::
GetQueryDescriptorInterface(void)

{
  return m_QueryDescriptorInterface;
}

//----------------------------------------------------------------------------
int  DummyTypeSearchTool::
SetQueryDescriptorInterface(DummyTypeDescriptorInterfaceABC
		       *aQueryDummyTypeDescriptorInterface)

{
  /* check if new value is different from old value*/
  if (m_QueryDescriptorInterface == 
      aQueryDummyTypeDescriptorInterface) return 0;
	
  /* release old interface*/
  if (m_QueryDescriptorInterface) 
    m_QueryDescriptorInterface->release();

  /* add new interface*/
  m_QueryDescriptorInterface = 
    aQueryDummyTypeDescriptorInterface;
  if (m_QueryDescriptorInterface) {
    m_QueryDescriptorInterface->addref();

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
    /* conect sub descriptors with sub Searchs*/
    if ((m_QueryDescriptorInterface->
	 GetSubDummyTypeADescriptorInterface()) &&
	m_SubDummyTypeASearch) {
      m_SubDummyTypeASearch->
	SetQueryDescriptorInterface(m_QueryDescriptorInterface->
			       GetSubDummyTypeADescriptorInterface());
    }
    if ((m_QueryDescriptorInterface->
	 GetSubDummyTypeBDescriptorInterface()) &&
	m_SubDummyTypeBSearch) {
      m_SubDummyTypeBSearch->
	SetQueryDescriptorInterface(m_QueryDescriptorInterface->
			       GetSubDummyTypeBDescriptorInterface());
    }
#endif /* __HasSubTypes*/
  }
  else {
#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
    /* if no descriptor, release also descriptor interfaces from
       sub Searchs*/
    m_SubDummyTypeASearch->SetQueryDescriptorInterface(0);
    m_SubDummyTypeBSearch->SetQueryDescriptorInterface(0);
#endif /* __HasSubTypes*/
    return -1;
  }

  return 0;
}

//----------------------------------------------------------------------------
double DummyTypeSearchTool::GetDistance(void)
{
  // ------------------------- Check it's all set up ok ---------------------
  if(m_RefDescriptorInterface==NULL)   return(DBL_MAX);
  if(m_QueryDescriptorInterface==NULL) return(DBL_MAX);

  if(strcmp(m_RefDescriptorInterface->GetName(),
	    "DummyTypeDescriptionInterface") != 0) return(DBL_MAX);
  if(strcmp(m_QueryDescriptorInterface->GetName(),
	    "DummyTypeDescriptionInterface") != 0) return(DBL_MAX);

  // ------------------------- Calculate the distance ------------------------
  // For this dummy type all I want is the abs difference between the values
  const long ref_val=m_RefDescriptorInterface->GetDummyContents();
  const long qur_val=m_QueryDescriptorInterface->GetDummyContents();

  double dist=static_cast<double> (ref_val-qur_val);
  if(dist<0.0) dist=-dist;

  // All done
  return(dist);
}

//----------------------------------------------------------------------------
DummyTypeSearchInterfaceABC *DummyTypeSearchTool::GetInterface(void)
{ return &m_Interface; }

